#!/usr/bin/python
#
# Use this script to calculate new arepo state.
# It takes 3 parameters: <32-bit rpmdir> <64-bit rpmdir> [<configfile>]
# and produces the output listing in the following format:
# 'i586-packagename EVR i586-packagename-VR.i586.rpm packagename-VR.i586.rpm arepo-mode'
#
# The config file [defaults to /etc/rpmrebuild-arepo.conf) contains a list of
# following regexps (egrep/python):
# Programs to arepo (prog_packages)
# - Files to repack in 'lib' mode (lib_files)
# - Files to determine 'prog' (prog_files)
# - Files to skip (blacklist_files)
# - Blacklist packages names (blacklist_pkg_name)
# - Blacklist packages with following content (blacklist_pkg_contents)

import rpm
import os
import sys
import re

class PackageHeader:
    def __init__(self, path):
        fd = os.open(path,os.O_RDONLY)
        self.hdr = rpm.headerFromPackage(fd)[0]
        self.name = self.hdr[rpm.RPMTAG_NAME]
        self.vr=self.hdr[rpm.RPMTAG_VERSION]+"-"+self.hdr[rpm.RPMTAG_RELEASE]
        if self.hdr[rpm.RPMTAG_EPOCH]!=None:
            self.evr=str(self.hdr[rpm.RPMTAG_EPOCH])+":"+self.vr
        else:
            self.evr=self.vr
        os.close(fd)

    def printself(self,mode):
        print "i586-"+self.name+"\t"+self.evr+"\ti586-"+self.name+"-"+self.vr+".i586.rpm\t"+self.name+"-"+self.vr+".i586.rpm\t"+mode

    def is_arepable(self, list64, filelist, blacklist_files, blacklist_pkg_contents):
        has_arepable_content = False
	for i in range(0,len(self.hdr[rpm.RPMTAG_FILENAMES])-1):
            fname = self.hdr[rpm.RPMTAG_FILENAMES][i]
            fflags = self.hdr[rpm.RPMTAG_FILEFLAGS][i]
            # In 'lib' mode
            if not has_arepable_content \
	        and not fname in list64 \
	        and not (fflags & rpm.RPMFILE_GHOST) \
	        and filelist.search(fname) \
	        and not blacklist_files.search(fname):
                # Will not intersect with 64-bit package, matches lib_files/prog_files and not matches blacklist_files
                has_arepable_content = True
            if blacklist_pkg_contents.search(fname):
                # Package has blacklisted content
                return False
        return has_arepable_content

if __name__ == "__main__":
    if len(sys.argv) < 3:
        sys.stderr.write("Usage: "+sys.argv[0]+" <32-bit rpmdir> <64-bit rpmdir> [<configfile>]\n")
        sys.exit(1)

    rpmdir32 = sys.argv[1]
    rpmdir64 = sys.argv[2]
    if not os.path.isdir(rpmdir32):
        sys.stderr.write("32-bit rpm dir invalid: "+rpmdir32+"!\n")
        sys.exit(1)
    if not os.path.isdir(rpmdir64):
        sys.stderr.write("64-bit rpm dir invalid: "+rpmdir64+"!\n")
        sys.exit(1)

    configfile = "/etc/rpmrebuild-arepo.conf"
    if len(sys.argv) > 3:
        configfile = sys.argv[3]

    fd = open(configfile,"r")

    prog_packages = ""
    lib_files = ""
    prog_files = ""
    blacklist_files = ""
    blacklist_pkg_name = ""
    blacklist_pkg_contents = ""

    for line in fd.readlines():
        if line.strip()=='' or line.strip().startswith('#'):
            continue
        (option,value) = line.split('=')
        (option,value) = (option.strip(' "\'\t'),value.strip(' "\'\t\n'))
        if option == "prog_packages":
            prog_packages = re.compile(value)
        elif option == "lib_files":
            lib_files = re.compile(value)
        elif option == "prog_files":
            prog_files = re.compile(value)
        elif option == "blacklist_files":
            blacklist_files = re.compile(value)
        elif option == "blacklist_pkg_name":
            blacklist_pkg_name = re.compile(value)
        elif option == "blacklist_pkg_contents":
            blacklist_pkg_contents = re.compile(value)
        else:
            sys.stderr.write("Unknown option in config: "+option+"!\n")
            sys.exit(1)

    rpms32 = os.listdir(rpmdir32)
    for rpm32 in rpms32:
        header = PackageHeader(rpmdir32+'/'+rpm32)
        if header.hdr[rpm.RPMTAG_ARCH] != 'i586':
            continue
        if blacklist_pkg_name.search(header.name):
            continue
        rpm64 = rpmdir64+'/'+header.name+'-'+header.vr+'.x86_64.rpm'
        if prog_packages.search(header.name):
            if header.is_arepable([], prog_files, blacklist_files, blacklist_pkg_contents):
                header.printself('prog')
        elif not os.path.isfile(rpm64):
            if header.is_arepable([], prog_files, blacklist_files, blacklist_pkg_contents):
                header.printself('prog')
            elif header.is_arepable([], lib_files, blacklist_files, blacklist_pkg_contents):
                header.printself('lib')
        else:
            header64 = PackageHeader(rpm64)
            if header.is_arepable(header64.hdr[rpm.RPMTAG_FILENAMES], lib_files, blacklist_files, blacklist_pkg_contents):
                header.printself('lib')
